@@ -50,6 +50,7 @@ TEST_P(LinkTestVulkan, FromFile)
     const size_t fileCount = fileNames.size();
     const EShMessages controls = DeriveOptions(Source::GLSL, Semantics::Vulkan, Target::AST);
     GlslangResult result;
+    result.validationResult = false;
 
     // Compile each input shader file.
     bool success = true;
@@ -108,6 +109,16 @@ INSTANTIATE_TEST_CASE_P(
     ::testing::ValuesIn(std::vector<std::vector<std::string>>({
         {"link1.vk.frag", "link2.vk.frag"},
         {"spv.unit1.frag", "spv.unit2.frag", "spv.unit3.frag"},
+		{"link.vk.matchingPC.0.0.frag", "link.vk.matchingPC.0.1.frag",
+			"link.vk.matchingPC.0.2.frag"},
+		{"link.vk.differentPC.0.0.frag", "link.vk.differentPC.0.1.frag",
+			"link.vk.differentPC.0.2.frag"},
+		{"link.vk.differentPC.1.0.frag", "link.vk.differentPC.1.1.frag",
+			"link.vk.differentPC.1.2.frag"},
+        {"link.vk.pcNamingValid.0.0.vert", "link.vk.pcNamingValid.0.1.vert"},
+        {"link.vk.pcNamingInvalid.0.0.vert", "link.vk.pcNamingInvalid.0.1.vert"},
+        {"link.vk.multiBlocksValid.0.0.vert", "link.vk.multiBlocksValid.0.1.vert"},
+        {"link.vk.multiBlocksValid.1.0.geom", "link.vk.multiBlocksValid.1.1.geom"},
     }))
 );
 // clang-format on
